@@ -98,7 +98,7 @@ BEGIN {
     require Exporter;
 
     # set the version for version checking
-    our $VERSION   = 4.16;
+    our $VERSION   = 4.18;
     our @ISA       = qw(Exporter);
     our @EXPORT    = qw();
     our @EXPORT_OK = qw();
@@ -182,6 +182,9 @@ sub new {
         'AND_MODE'    => 3,
         'MASK_MODE'   => 4,
         'CLIPPED'     => 0,
+        'ARC'         => 0,
+        'PIE'         => 1,
+        'POLY_ARC'    => 2,
 
         ## Set up the Framebuffer driver "constants" defaults
         # Commands
@@ -208,7 +211,8 @@ sub new {
         'FBinfo_hwaccel_imageblit' => 'I6C1I2',          # dx(32),dy(32),width(32),height(32),fg_color(32),bg_color(32),depth(8),image pointer(32),color map pointer(32)
                                                          # COLOR MAP:
                                                          #  start(32),length(32),red(16),green(16),blue(16),alpha(16)
-                                                         # Default values
+
+        # Default values
         'VXRES'                    => 640,
         'VYRES'                    => 480,
         'BITS'                     => 32,
@@ -258,7 +262,6 @@ sub new {
         # Now that everything is set up, let's map the framebuffer to SCREEN
 
         mmap($self->{'SCREEN'},  $self->{'smem_len'}, PROT_READ | PROT_WRITE, MAP_SHARED, $self->{'FB'});
-        mmap($self->{'SCREEN2'}, $self->{'smem_len'}, PROT_READ | PROT_WRITE, MAP_SHARED, $self->{'FB'});
     } else {    # Go into emulation mode if no actual framebuffer available
                 # Aww bummer, no screen.  Ok, turn on emulation mode then
         $self->{'ERROR'} = 'Framebuffer Device Not Found! Emulation mode.  EXPERIMENTAL!!';
@@ -797,7 +800,7 @@ sub drawto {
 
 =head2 draw_arc
 
-Draws an arc of a circle at point x,y.
+Draws an arc/pie/poly arc of a circle at point x,y.
 
 =over 1
 
@@ -851,30 +854,28 @@ sub draw_arc {
 
     my $mode = int($params->{'mode'}       || 0);
     my $size = int($params->{'pixel_size'} || 1);
-    $size = 1 if ($mode == 1);
+#    $size = 2 if ($mode == $self->{'PIE'});
     my ($sx, $sy, $degrees, $ox, $oy) = (0, 0, 1, 1, 1);
     $self->{'history'} = {};    # Initialize the history buffer for optimization and xor beautifying.
 
     $degrees = $start_degrees;
     if ($start_degrees > $end_degrees) {
         do {
-            $sx = $x - ($radius * sin(($degrees * pi) / 180));
-            $sy = $y - ($radius * cos(($degrees * pi) / 180));
+            $sx = int($x - ($radius * sin(($degrees * pi) / 180)));
+            $sy = int($y - ($radius * cos(($degrees * pi) / 180)));
             if (($sx <=> $ox) || ($sy <=> $oy)) {
                 switch ($mode) {
-                    case (0) {     # Ordinary arc
+                    case ($self->{'ARC'}) {     # Ordinary arc
                         $self->plot({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
                     }
-                    case (1) {     # Filled arc
-                        $self->plot({'x' => $x, 'y' => $y, 'pixel_size' => $size});
-                       $self->drawto({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
+                    case ($self->{'PIE'}) {     # Filled arc
+                        $self->line({'x' => $x, 'y' => $y, 'xx' => $sx, 'yy' => $sy, 'pixel_size' => $size});
                     }
-                    case (2) {     # Poly arc
+                    case ($self->{'POLY_ARC'}) {     # Poly arc
                         if ($degrees == $start_degrees) {
-                            $self->plot({'x' => $x, 'y' => $y, 'pixel_size' => $size});
-                            $self->drawto({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
+                            $self->line({'x' => $x, 'y' => $y, 'xx' => $sx, 'yy' => $sy, 'pixel_size' => $size});
                         } else {
-                            $self->plot({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
+                            $self->drawto({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
                         }
                     } ## end case (2)
                 } ## end switch
@@ -886,23 +887,21 @@ sub draw_arc {
         $degrees = 0;
     } ## end if ($start_degrees > $end_degrees)
     do {
-        $sx = $x - ($radius * sin(($degrees * pi) / 180));
-        $sy = $y - ($radius * cos(($degrees * pi) / 180));
+        $sx = int($x - ($radius * sin(($degrees * pi) / 180)));
+        $sy = int($y - ($radius * cos(($degrees * pi) / 180)));
         if (($sx <=> $ox) || ($sy <=> $oy)) {
             switch ($mode) {
-                case (0) {    # Ordinary arc
+                case ($self->{'ARC'}) {    # Ordinary arc
                     $self->plot({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
                 }
-                case (1) {    # Filled arc
-                    $self->plot({'x' => $x, 'y' => $y, 'pixel_size' => $size});
-                    $self->drawto({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
+                case ($self->{'PIE'}) {    # Filled arc
+                    $self->line({'x' => $x, 'y' => $y, 'xx' => $sx, 'yy' => $sy, 'pixel_size' => $size});
                 }
-                case (2) {    # Poly arc
+                case ($self->{'POLY_ARC'}) {    # Poly arc
                     if ($degrees == $start_degrees) {
-                        $self->plot({'x' => $x, 'y' => $y, 'piel_size' => $size});
-                        $self->drawto({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
+                        $self->line({'x' => $x, 'y' => $y, 'xx' => $sx, 'yy' => $sy, 'pixel_size' => $size});
                     } else {
-                        $self->plot({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
+                        $self->drawto({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
                     }
                 } ## end case (2)
             } ## end switch
@@ -911,14 +910,133 @@ sub draw_arc {
         } ## end if (($sx <=> $ox) || (...))
         $degrees += $granularity;
     } until ($degrees >= $end_degrees);
-    if ($mode == 2) {
-        $self->plot({'x' => $x, 'y' => $y, 'pixel_size' => $size});
-        $self->drawto({'x' => $sx, 'y' => $sy, 'pixel_size' => $size});
+    if ($mode == $self->{'POLY_ARC'}) {
+        $self->line({'x' => $x, 'y' => $y, 'xx' => $sx, 'yy' => $sy, 'pixel_size' => $size});
     }
-    $self->_fill_history() if ($mode == 2);
+#    $self->_fill_history() if ($mode == $self->{'PIE'});
     delete($self->{'history'}) if (exists($self->{'history'}));    # Get rid of the history buffer. So it doesn't mess anything up.
 } ## end sub draw_arc
 
+=head2 arc
+
+Draws an arc of a circle at point x,y.  This is an alias to draw_arc above,
+but no mode parameter needed.
+
+=over 1
+
+ x             = x of center of circle
+ y             = y of center of circle
+ radius        = radius of circle
+ start_degrees = starting point, in degrees, of arc
+ end_degrees   = ending point, in degrees, of arc
+ granularity   = This is used for accuracy in drawing
+                 the arc.  The smaller the number, the
+                 more accurate the arc is drawn, but it
+                 is also slower.  Values between 0.1
+                 and 0.01 are usually good.  Valid values
+                 are any positive floating point number
+                 down to 0.0001.
+
+ $fb->arc({
+    'x'             => 100,
+    'y'             => 100,
+    'radius'        => 100,
+    'start_degrees' => -40,
+    'end_degrees'   => 80,
+    'grandularity   => .05,
+ });
+
+=back
+
+=cut
+
+sub arc {
+    my $self   = shift;
+    my $params = shift;
+    $params->{'mode'} = $self->{'ARC'};
+    $self->draw_arc($params);
+}
+
+=head2 filled_pie
+
+Draws a filled pie wedge at point x,y.  This is an alias to draw_arc above,
+but no mode parameter needed.
+
+=over 1
+
+ x             = x of center of circle
+ y             = y of center of circle
+ radius        = radius of circle
+ start_degrees = starting point, in degrees, of arc
+ end_degrees   = ending point, in degrees, of arc
+ granularity   = This is used for accuracy in drawing
+                 the arc.  The smaller the number, the
+                 more accurate the arc is drawn, but it
+                 is also slower.  Values between 0.1
+                 and 0.01 are usually good.  Valid values
+                 are any positive floating point number
+                 down to 0.0001.
+
+ $fb->filled_pie({
+    'x'             => 100,
+    'y'             => 100,
+    'radius'        => 100,
+    'start_degrees' => -40,
+    'end_degrees'   => 80,
+    'grandularity   => .05,
+ });
+
+=back
+
+=cut
+
+sub filled_pie {
+    my $self   = shift;
+    my $params = shift;
+    $params->{'mode'} = $self->{'PIE'};
+    $self->draw_arc($params);
+}
+
+=head2 poly_arc
+
+Draws a poly arc of a circle at point x,y.  This is an alias to draw_arc above,
+but no mode parameter needed.
+
+=over 1
+
+ x             = x of center of circle
+ y             = y of center of circle
+ radius        = radius of circle
+ start_degrees = starting point, in degrees, of arc
+ end_degrees   = ending point, in degrees, of arc
+ granularity   = This is used for accuracy in drawing
+                 the arc.  The smaller the number, the
+                 more accurate the arc is drawn, but it
+                 is also slower.  Values between 0.1
+                 and 0.01 are usually good.  Valid values
+                 are any positive floating point number
+                 down to 0.0001.
+
+ $fb->poly_arc({
+    'x'             => 100,
+    'y'             => 100,
+    'radius'        => 100,
+    'start_degrees' => -40,
+    'end_degrees'   => 80,
+    'grandularity   => .05,
+ });
+
+=back
+
+=cut
+
+sub poly_arc {
+    my $self   = shift;
+    my $params = shift;
+    $params->{'mode'} = $self->{'POLY_ARC'};
+    $self->draw_arc($params);
+}
+
 =head2 ellipse
 
 Draw an ellipse at center position x,y with XRadius, YRadius.  Either a filled
@@ -935,6 +1053,7 @@ Clipping Applies.
     'xradius'    => 50,
     'yradius'    => 100,
     'filled'     => 0,
+    'factor'     => 1, # Anything other than 1 has funkiness
     'pixel_size' => 4
  });
 
@@ -982,19 +1101,19 @@ sub ellipse {
         my $cyy  = $cy + $y;
         my $cy_y = $cy - $y;
         if ($filled) {
-            if ($cyy <=> $old_cyy) {
+            if (int($cyy) <=> $old_cyy) {
                 $self->line({'x' => $cxx, 'y' => $cyy,'xx' => $cx_x, 'yy' => $cyy});
                 $old_cyy = int($cyy);
             }
-            if (($cy_y <=> $old_cy_y) && ($cyy <=> $cy_y)) {
+            if ((int($cy_y) <=> $old_cy_y) && (int($cyy) <=> int($cy_y))) {
                 $self->line({'x' => $cx_x, 'y' => $cy_y,'xx' => $cxx, 'yy' => $cy_y});
                 $old_cy_y = int($cy_y);
             }
         } else {
             $self->plot({'x' => $cxx,  'y' => $cyy,  'pixel_size' => $size});
             $self->plot({'x' => $cx_x, 'y' => $cyy,  'pixel_size' => $size});
-            $self->plot({'x' => $cx_x, 'y' => $cy_y, 'pixel_size' => $size}) if ($cyy <=> $cy_y);
-            $self->plot({'x' => $cxx,  'y' => $cy_y, 'pixel_size' => $size}) if ($cyy <=> $cy_y);
+            $self->plot({'x' => $cx_x, 'y' => $cy_y, 'pixel_size' => $size}) if (int($cyy) <=> int($cy_y));
+            $self->plot({'x' => $cxx,  'y' => $cy_y, 'pixel_size' => $size}) if (int($cyy) <=> int($cy_y));
         } ## end else [ if ($filled) ]
         $y++;
         $StoppingY    += $TwoASquare;
@@ -1023,18 +1142,18 @@ sub ellipse {
         my $cyy  = $cy + $y;
         my $cy_y = $cy - $y;
         if ($filled) {
-            if ($cyy <=> $old_cyy) {
+            if (int($cyy) <=> $old_cyy) {
                 $self->line({'x' => $cxx, 'y' => $cyy, 'xx' => $cx_x, 'yy' => $cyy});
                 $old_cyy = int($cyy);
             }
-            if (($cy_y <=> $old_cy_y) && ($cyy <=> $cy_y)) {
+            if ((int($cy_y) <=> $old_cy_y) && (int($cyy) <=> int($cy_y))) {
                 $self->line({'x' => $cx_x, 'y' => $cy_y, 'xx' => $cxx, 'yy' => $cy_y});
                 $old_cy_y = int($cy_y);
             }
         } else {
             $self->plot({'x' => $cxx,  'y' => $cyy,  'pixel_size' => $size});
-            $self->plot({'x' => $cx_x, 'y' => $cyy,  'pixel_size' => $size}) if ($cxx <=> $cx_x);
-            $self->plot({'x' => $cx_x, 'y' => $cy_y, 'pixel_size' => $size}) if ($cxx <=> $cx_x);
+            $self->plot({'x' => $cx_x, 'y' => $cyy,  'pixel_size' => $size}) if (int($cxx) <=> int($cx_x));
+            $self->plot({'x' => $cx_x, 'y' => $cy_y, 'pixel_size' => $size}) if (int($cxx) <=> int($cx_x));
             $self->plot({'x' => $cxx,  'y' => $cy_y, 'pixel_size' => $size});
         } ## end else [ if ($filled) ]
         $x++;
@@ -2223,7 +2342,7 @@ __END__
 
 The module can NOT have separate threads calling the same object.
 You WILL crash. However, you can instantiate an object for each
-thread and it will work just fine!
+thread to use, and it will work just fine!
 
 See the "examples" directory for "frtest.pl" as an example of a
 threading script that uses this module.  Just add the number of
@@ -2240,16 +2359,17 @@ tremendously.
 Horizontal lines and filled boxes draw very fast.  Learn to
 exploit them.
 
-Filled boxes draw faster than thick horizontal lines.  See
-where I'm going?
-
 =head2 PIXEL SIZE
 
-Pixel sizes over 1 utilize a filled "circle" to do the drawing.
-This is why the larger the "pixel", the slower the draw.  Maybe
-for speed sake, a loop of lines drawn at single pixel size, but
-offset to make a "wide" line, will draw faster for you.  If making
-a game, or user interface, you may want to consider this technique.
+Pixel sizes over 1 utilize a filled "box" or "circle" (negative
+numbers for circle) to do the drawing.  This is why the larger
+the "pixel", the slower the draw.
+
+=head2 PIE ARCS
+
+When drawing a filled pie arc, try larger pixel sizes, in
+addition to tweaking the granularity.  Larger pixel sizes will
+fill in gaps as a result of higher granularities.
 
 =head1 AUTHOR
 
@@ -2264,7 +2384,7 @@ modify it under the same terms as Perl itself.
 
 =head1 VERSION
 
-Version 4.16 (July 14, 2015)
+Version 4.18 (July 14, 2015)
 
 =head1 THANKS
 
